//
//  ABSViewController.m
//  AudioPlayerExample
//
//  Created by Mark Gregory on 05/11/2012.
//  Copyright (c) 2012 Abbey Software Development Limited. All rights reserved.
//

/*
 NB This only works properly on device it does not work on the simulator correctly.
 when in the simulator you need to add a backgrund call to ensure that the audio 
 gets played in the background but it does not register with the events.
 We use normal notificaitons for the volume control which works well and tracks the 
 volume level exactly.
 */


#import "ABSViewController.h"
#import <AudioToolbox/AudioToolbox.h>

@interface ABSViewController ()
{
    UIBackgroundTaskIdentifier bgTaskId;
}
@property (nonatomic,strong)AVAudioPlayer *audioPlayer;

@property (nonatomic,strong) UIButton *playButton;
@property (nonatomic,strong) UIButton *stopButton;
@property (nonatomic,strong) UISlider *volume;
@property (nonatomic,strong) UISlider *scrubber;
@property (nonatomic,strong) UIProgressView *currentPosition;
@property (nonatomic,strong) NSTimer *trackProgessTimer;
@property (nonatomic,assign) BOOL interrupted;

@end

@implementation ABSViewController

@synthesize audioPlayer;
@synthesize playButton;
@synthesize stopButton;
@synthesize currentPosition;
@synthesize scrubber;
@synthesize trackProgessTimer;
@synthesize volume;
@synthesize interrupted;

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    NSURL *fileURL = [NSURL fileURLWithPath:[[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:@"song.mp3"]];
    NSError *error = nil;
  
  
    
    // Enable Background Operation
    // also ensure that you set the RequiredBackGroundModes to App Plays Audio in the
    // apps info.plist
    
    OSStatus status = AudioSessionInitialize(NULL, NULL, NULL, NULL);
    UInt32 sessionCategory = kAudioSessionCategory_MediaPlayback;
    status = AudioSessionSetProperty (kAudioSessionProperty_AudioCategory, sizeof
                                        (sessionCategory),
                                        &sessionCategory);
    AudioSessionSetActive(YES);
    
    
    [[NSNotificationCenter defaultCenter]
     addObserver:self
     selector:@selector(volumeChanged:)
     name:@"AVSystemController_SystemVolumeDidChangeNotification"
     object:nil];
    
       
    audioPlayer = [[AVAudioPlayer alloc] initWithContentsOfURL:fileURL error:&error];
    [audioPlayer setDelegate:self];
    
    if (error)
      NSLog(@"Error: %@",error);
    else
      [audioPlayer prepareToPlay];
   
    
    [[UIApplication sharedApplication] beginReceivingRemoteControlEvents];
    [self createUI];

}


-(void)viewDidAppear:(BOOL)animated{
    [super viewDidAppear:animated];
    [self becomeFirstResponder];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}


- (BOOL)canBecomeFirstResponder {
    return YES;
}


-(void)dealloc{
    [audioPlayer release];
    [[UIApplication sharedApplication] endReceivingRemoteControlEvents];
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    AudioSessionSetActive(NO);
    [super dealloc];
}


-(void)createUI{
    playButton=[UIButton buttonWithType:UIButtonTypeRoundedRect];
    [playButton setFrame:CGRectMake(60, 10, 80, 34)];
    [playButton setTitle:@"Play" forState:UIControlStateNormal];
    [playButton addTarget:self action:@selector(playTrack:) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:playButton];
    [playButton release];
    
    stopButton=[UIButton buttonWithType:UIButtonTypeRoundedRect];
    [stopButton setFrame:CGRectMake(180, 10, 80, 34)];
    [stopButton setTitle:@"Stop" forState:UIControlStateNormal];
    [stopButton addTarget:self action:@selector(stopTrack:) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:stopButton];
    [stopButton release];
    
    volume=[[UISlider alloc] initWithFrame:CGRectMake(60, 80, 200, 0)];
    [volume addTarget:self action:@selector(changeVolume:) forControlEvents:UIControlEventValueChanged];
    [volume setMinimumValue:0.0];
    [volume setMaximumValue:1.0];
    [volume setValue:[audioPlayer volume]];
    [volume setContinuous:YES];
    [self.view addSubview:volume];
    [volume release];
    
    
    scrubber=[[UISlider alloc] initWithFrame:CGRectMake(60, 140, 200, 0)];
    [scrubber addTarget:self action:@selector(scrubTrack:) forControlEvents:UIControlEventValueChanged];
    [scrubber setMinimumValue:0.0];
    [scrubber setMaximumValue:[audioPlayer duration]];
    [scrubber setValue:[audioPlayer currentTime]];
    [scrubber setContinuous:YES];
    [self.view addSubview:scrubber];
    [scrubber release];
    
    
}


-(void)updateScrubber:(id)sender{
    [scrubber setValue:(float)[audioPlayer currentTime]];
}


-(void)playTrack:(id)sender{
    
    if (! audioPlayer.playing)
    {
      [audioPlayer play];
      [playButton setTitle:@"Pause" forState:UIControlStateNormal];
      trackProgessTimer=[NSTimer scheduledTimerWithTimeInterval:1.0
                                                       target:self
                                                     selector:@selector(updateScrubber:)
                                                     userInfo:nil
                                                      repeats:YES];

    }
    else
    {
      [trackProgessTimer invalidate];
      [audioPlayer pause];
      [playButton setTitle:@"Play" forState:UIControlStateNormal];
    }
    
    [self becomeFirstResponder];
    
}


-(void)stopTrack:(id)sender{
    [audioPlayer stop];
    [audioPlayer setCurrentTime:0];
    [playButton setTitle:@"Play" forState:UIControlStateNormal];
    [self becomeFirstResponder];
}


-(void)changeVolume:(id)sender{
    [audioPlayer setVolume:[volume value]];
    [self becomeFirstResponder];
}


-(void)scrubTrack:(id)sender{
  if([audioPlayer isPlaying])
    [audioPlayer setCurrentTime:[scrubber value]];
  else
    [audioPlayer setCurrentTime:[scrubber value]];
  [self becomeFirstResponder];

}


-(void)resetControls{
    [self becomeFirstResponder];
    [audioPlayer setCurrentTime:0.0];
    [scrubber setValue:0.0];
    [trackProgessTimer invalidate];
    [playButton setTitle:@"Play" forState:UIControlStateNormal];
}





#pragma mark Audio Player Delegate

-(void)audioPlayerBeginInterruption:(AVAudioPlayer *)player{
  interrupted=[audioPlayer isPlaying];
  [trackProgessTimer invalidate];
}

-(void)audioPlayerEndInterruption:(AVAudioPlayer *)player withOptions:(NSUInteger)flags{
    if(interrupted)
         [audioPlayer play];
}


- (void)audioPlayerDidFinishPlaying:(AVAudioPlayer *)player successfully:(BOOL)success
{
    [self resetControls];
}


- (void)volumeChanged:(NSNotification *)notification
{
    float trackVolume =[[[notification userInfo]
      objectForKey:@"AVSystemController_AudioVolumeNotificationParameter"]
     floatValue];
    
    [audioPlayer setVolume:trackVolume];
    [volume setValue:trackVolume];
}


-(void)remoteControlReceivedWithEvent:(UIEvent *)event{
    NSLog(@"Control Events");
    
    switch ([event subtype])
    {
      case UIEventSubtypeRemoteControlTogglePlayPause:
        [self playTrack:nil];
        break;
      case UIEventSubtypeRemoteControlNextTrack:
        break;
      case UIEventSubtypeRemoteControlBeginSeekingForward:
        break;
      case UIEventSubtypeRemoteControlPlay:
        [self playTrack:self];
        break;
      case UIEventSubtypeRemoteControlStop:
        [self stopTrack:self];
        break;
      case UIEventSubtypeRemoteControlPause:
        [self playTrack:self];
        break;
      case UIEventSubtypeRemoteControlBeginSeekingBackward:
        break;
      case UIEventSubtypeRemoteControlPreviousTrack:
        break;
      case UIEventSubtypeMotionShake:
        break;
      case UIEventSubtypeRemoteControlEndSeekingBackward:
        break;
      case UIEventSubtypeRemoteControlEndSeekingForward:
        break;
      case UIEventSubtypeNone:
        break;
      default:
          break;
    }
}


@end
